#!/usr/bin/python
# watch-bodhi-requests.py - script to run periodically and launch autoqa tests
#                           when new requests appear in bodhi.
#
# Copyright 2010, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#     Will Woods <wwoods@redhat.com>

# Overall strategy:
# List newly-created updates, and put all the ones that have requested to be
# moved to testing or stable into the list of pending updates.
#
# Notes:
#
# Here's the deal - we need to check for two different classes of updates.
#   1) newly-created updates: i.e. anything where date_submitted >= last_check
#   2) updates moving from 'testing' to 'stable'
#
# There's a wrinkle with the former case: critpath updates start with
# 'request':None, so we don't know what repo they're going to.
# Since this watcher is supposed to be a temporary solution anyway, we decided
# that the least bad solution is just to fire tests against both repos and let
# the human testers decide which results are important.
#
# The latter class is the trickier one. To handle this we can look at the set
# of updates where {'request':'stable', 'status':'testing'}. Unfortunately
# requesting 'stable' does not change the date_modified timestamp, so we can't
# filter on that to find only *new* requests.
# What we do instead is fire a test the first time we see a stable update,
# and write their update_id to a list of previously-seen stable updates.
# Thus we can filter out previously-seen updates on future runs.
#
# N.B.: if an update gets pushed to testing or stable *before* we poll and
# notice it, we'll miss it. But then, if stuff is getting pushed before we
# manage to run our tests, then rel-eng probably had a damn good reason.


import fedora.client
import optparse
import time
import os
from autoqa.repoinfo import repoinfo
from autoqa.bodhi_utils import bodhitime, parse_bodhitime, bodhi_list

cachedir = '/var/cache/autoqa/watch-bodhi-requests'
try:
    os.makedirs(cachedir)
except OSError, e:
    if e.errno != 17: # already exists
        raise

# XXX we should have actual repoinfo data for these
#bodhi_releases = [r['bodhi_release'] for r in repoinfo.repos()
#                  if 'bodhi_release' in r]
bodhi_releases = ['F11','F12','F13']
archlist = ('i686', 'x86_64', 'noarch')

# XXX NOTE we could probably be using cPickle or something here, rather than
# a possibly-unreliable invented encoding/decoding to save a set()
def save_update_ids(update_ids, cachefile):
    try:
        f = open(cachefile,'w')
        for u in update_ids:
            f.write(u + "\n")
        f.close()
    except IOError, e:
        print "error: failed to write %s: %s" % (cachefile, str(e))
def load_update_ids(cachefile):
    update_ids = set()
    try:
        f = open(cachefile)
        update_ids = set([line.strip() for line in f])
        f.close()
    except IOError, e:
        print "warning: failed to read %s: %s" % (cachefile, str(e))
    return update_ids

# Do a bodhi_list and filter out anything listed as previously-seen
def bodhi_list_and_filter(params, cachefile, updatecache=True):
    '''Gets a list of updates in bodhi that match the given params, then
    checks the given cachefile for updates it has seen before. Returns a list
    of only the new updates, and updates the cachefile.'''
    # list updates that match params
    updates = bodhi_list(params)
    # remove any update that shows up in the cachefile
    current_ids = set([u['title'] for u in updates])
    old_ids = load_update_ids(cachefile)
    new_ids = current_ids.difference(old_ids)
    # update cachefile
    if updatecache:
        save_update_ids(current_ids, cachefile)
    # return the filtered list
    return [u for u in updates if u['title'] in new_ids]

def bodhi_new_requests_since(release, timestamp, cachefile=None, updatecache=True):
    '''List newly-created bodhi requests - that is, bodhi requests that have
    been created since the given timestamp.'''
    if cachefile is None:
        cachefile = os.path.join(cachedir,"%s-testing-request-ids" % release)
    # created_since is an undocumented argument to bodhi's list call
    # see also: start_date, end_date, pushed_since
    params = {'release': release,
              'status': 'pending',
              'created_since': bodhitime(timestamp)}
    # N.B. this may return updates with 'request': None, which may need to be
    # handled specially by the calling method.
    return bodhi_list_and_filter(params, cachefile, updatecache)

def bodhi_new_stable_requests(release, cachefile=None, updatecache=True):
    '''List new bodhi requests for stable - 'new' in this case meaning
    'new since the last time we checked'. Basically this lists all the
    updates that have requested to move from testing to stable and filters
    out any update that we've already seen (so we don't retest the same
    updates over and over.'''
    if cachefile is None:
        cachefile = os.path.join(cachedir,"%s-stable-request-ids" % release)
    params = {'release': release,
              'status': 'testing',
              'request': 'stable'}
    return bodhi_list_and_filter(params, cachefile, updatecache)

def self_test():
    age = 5
    release='F13'
    updates = []
    while len(updates) == 0:
        print "Looking for new update requests in the past %i minutes" % age
        updates = bodhi_new_requests_since(release, int(time.time() - age*60))
        age += 10
        if age > 150:
            print "Augh, I give up."
            break
    for u in updates:
        print "  %s (%s)" % (u['title'], u['request'])
    print
    # TODO simulate straight-to-stable, critpath

    print "Looking for testing->stable requests"
    cachefile = '/tmp/bodhi-watcher-self-test.cache'
    try:
        os.remove(cachefile)
        f = open(cachefile,'w')
        f.close()
    except OSError:
        pass
    stable_updates = bodhi_new_stable_requests(release, cachefile)
    for u in stable_updates:
        print "  %s (%s)" % (u['title'], u['request'])
    print "Removing last update from cache file (to simulate new update)"
    stable_update_ids = [u['updateid'] for u in stable_updates]
    stable_update_ids.pop()
    save_update_ids(stable_update_ids, cachefile)
    print "Looking for 'new' testing->stable request"
    stable_updates = bodhi_new_stable_requests(release, cachefile)
    for u in stable_updates:
        print "  %s (%s)" % (u['title'], u['request'])

def main():
    parser = optparse.OptionParser(description='Script to kick off autoqa tests\
when new requests are filed in bodhi')
    parser.add_option('--dryrun','--dry-run',action='store_true',
        help='Do not actually execute commands, just show what would be done')
    parser.add_option('--selftest','--self-test',action='store_true',
        help='Run a quick self-test')
    (opts, args) = parser.parse_args()

    if opts.selftest:
        self_test()
        return

    arglist = []
    # TODO allow CLI override of bodhi_releases
    for r in bodhi_releases:
        testing_cachefile = os.path.join(cachedir,"%s-%s-update-ids" % (r, 'testing'))
        stable_cachefile = os.path.join(cachedir,"%s-%s-update-ids" % (r, 'stable'))

        print "Looking for new update requests for %s" % r
        try:
            lastcheck = os.stat(testing_cachefile).st_mtime
            print "lastcheck was %i seconds ago" % int(time.time() - lastcheck)
        except OSError:
            print "No cachefile present, checking for requests in last 30 min."
            lastcheck = time.time() - (30 * 60)
        new_updates = []
        try:
            new_updates = bodhi_new_requests_since(r, int(lastcheck), testing_cachefile,
                                                    updatecache=(not opts.dryrun))
        except fedora.client.ServerError, e:
            print "ERROR: %s" % str(e)
        # Sort the new updates based on their target/request
        updates = {'testing': [], 'stable': [], 'critpath': []}
        for u in new_updates:
            request = u['request'] or 'critpath' # None = critpath (see below)
            updates[request].append(u)
        print "%i new update requests (%i testing, %i stable, %i critpath)" % \
            (len(new_updates), len(updates['testing']), len(updates['stable']),
             len(updates['critpath']))

        print "Looking for requests moving from testing to stable for %s" % r
        new_updates = bodhi_new_stable_requests(r, stable_cachefile,
                                               updatecache=(not opts.dryrun))
        print "%i stable update requests found" % len(new_updates)
        updates['stable'] += new_updates

        # NOTE: as discussed in this QA meeting:
        #   https://fedoraproject.org/wiki/QA/Meetings/20100517#t15:40:57
        # it's possible that some of the test_updates will have
        # request=None (e.g. critpath updates). We have no way of knowing
        # whether these updates are going to end up in testing or stable,
        # so the safe (albeit kind of stupid) agreed-upon solution is to
        # test against both repos. Therefore:
        updates['testing'] += updates['critpath']
        updates['stable'] += updates['critpath']

        def add_update_to_arglist(u, tag_suffix):
            # either title or updateid is used to construct the bodhi URL
            cmdargs = ['--title', u['title']]
            if u['updateid']:
                cmdargs += ['--updateid', u['updateid']]
            cmdargs += ['--target-tag', u['release']['dist_tag']+tag_suffix]
            envrs = [b['nvr'] for b in u['builds']]
            # XXX LAME HACK: assume all arches for all builds
            # XXX LAME HACK 2: since autoqa doesn't handle 'noarch', skip it
            for arch in archlist:
                if arch is not 'noarch':
                    cmdargs += ['--arch', arch]
            cmdargs += envrs
            arglist.append(cmdargs)

        for u in updates['stable']:
            add_update_to_arglist(u, '-updates')
        for u in updates['testing']:
            add_update_to_arglist(u, '-updates-testing')

    for a in arglist:
        harnesscall = ['autoqa', 'post-bodhi-update'] + a
        if opts.dryrun:
            print " ".join(harnesscall)
        else:
            subprocess.call(harnesscall)

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        print
