#!/usr/bin/python
#
# watch-koji-builds.py - script to watch a set of koji tags for new builds and
#                        kick off tests when new builds/packages are found.
#
# Copyright 2010, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#     Will Woods <wwoods@redhat.com>

import os
import sys
import koji
import time
import pickle
import xmlrpclib
import subprocess
import optparse
from autoqa import koji_utils
from autoqa.repoinfo import repoinfo

# Directory where we cache info between runs
cachedir = '/var/cache/autoqa/watch-koji-builds'
cachefile = os.path.join(cachedir,"untagged")
try:
    os.makedirs(cachedir)
except OSError, e:
    if e.errno != 17: # already exists
        raise
# XXX configparser? /etc/koji.conf, section 'koji', item 'server'
#     alternately: read from e.g. /etc/autoqa/autoqa.conf
kojiserver = 'http://koji.fedoraproject.org/kojihub'

def get_prevtime():
    '''Returns the prevtime to use when looking for new builds.  Value will be
       either 1) the mtime of the cachefile, or 2) if no cachefile is found, current
       time - 3 hours.'''
    try:
        return os.stat(cachefile)[8] # mtime
    except:
        return time.time() - 10800

def load_untagged():
    '''Load the list of untagged builds seen by previous runs'''
    try:
        f = open(cachefile)
        untagged = pickle.load(f)
        f.close()
        return untagged
    except:
        return []

def save_list(pkglist):
    '''Save a list of builds for a subsequent run'''
    out = open(cachefile,"w")
    pickle.dump(pkglist, out)
    out.close()

def new_builds_since(session, taglist, prevtime):

    untagged_builds = []
    tagged_builds = {}
    for tag in taglist:
        tagged_builds[tag] = []
    builds = session.list_builds_since(prevtime)
    builds += load_untagged() # untagged builds from previous run
    for nvr in builds:
        tags = set(session.tag_history(nvr))
        if tags:
            # Only check packages with tags from the taglist
            for t in taglist.intersection(tags):
                tagged_builds[t].append(nvr)
        elif not session.build_failed(nvr):
            # No tag yet but the build is still OK - check again later
            untagged_builds.append(nvr)
    if opts.verbose:
        print "untagged: %s" % " ".join(untagged_builds)
        print "tagged: %s" % " ".join(tagged_builds)
    return (tagged_builds, untagged_builds)

if __name__ == '__main__':
    # Set up the option parser
    # TODO: optparse for prevtime
    parser = optparse.OptionParser(description='Script to watch a set of koji \
tags for new builds and kick off tests when new builds/packages are found.')
    parser.add_option('--dryrun', '--dry-run', action='store_true',
        help='Do not actually execute commands, just show what would be done')
    parser.add_option('-p', '--prevtime', action='store',
        type='float', default=get_prevtime(),
        help='How far back to look for new builds (seconds since last epoch)')
    parser.add_option('--verbose', action='store_true',
        help='Print extra information')
    (opts, args) = parser.parse_args()

    # Using repoinfo, establish the set of tags to look for
    taglist = set()
    for repo in repoinfo.repos():
        taglist.add(repoinfo.get(repo, 'tag'))

    if opts.verbose:
        print "Looking up builds since %s (%s)" % (opts.prevtime, time.ctime(opts.prevtime))
        print "Looking for builds with tags %s" % " ".join(taglist)

    # Set up the koji connection
    kojiopts = {} # Possible items: user, password, debug_xmlrpc, debug..
    session = koji_utils.SimpleKojiClientSession(kojiserver, kojiopts)
    untagged_builds = []
    tagged_builds = []
    try:
        session.ensure_connection()
        (tagged_builds, untagged_builds) = new_builds_since(session,
                                            taglist, opts.prevtime)
        if not opts.dryrun:
            save_list(untagged_builds)
        for tag, builds in tagged_builds.items():
            for b in builds:
                # Get a list of all package arches in this build
                arches = [r['arch'] for r in session.listRPMs(b['build_id'])]
                harnesscall = ['autoqa', 'post-koji-build',
                               '--name', b['name'],
                               '--kojitag', tag]

                # Invoke autoqa with an --arch arg for each arch we care about
                repoarches = set(repoinfo.getrepo_by_tag(tag).get("arches"))
                testarches = set(repoarches).intersection(arches)

                # NOTE HACK: autoqa doesn't handle 'noarch', use 'x86_64'
                if 'noarch' in arches:
                    testarches.discard('noarch')
                    testarches.add('x86_64')

                for arch in testarches:
                    harnesscall += ['--arch', arch]
                if b['epoch']:
                    envr = '%s:%s' % (b['epoch'], b['nvr'])
                else:
                    envr = b['nvr']
                harnesscall.append(envr)
                if opts.dryrun:
                    print " ".join(harnesscall)
                    continue
                subprocess.call(harnesscall)
    except KeyboardInterrupt:
        print "Exiting on keyboard interrupt."
