#
# koji_utils.py - helper routines for dealing with koji builds
#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import time
import koji
from repoinfo import repoinfo
import rpmUtils.miscutils

# XXX fetch from /etc/koji.conf, section 'koji'
kojiurl = 'http://koji.fedoraproject.org/kojihub'
pkgurl = 'http://koji.fedoraproject.org/packages'

class SimpleKojiClientSession(koji.ClientSession):
    '''Convenience wrapper class for interacting with koji'''
    def __init__(self, server=kojiurl, opts=None):
        return koji.ClientSession.__init__(self, server, opts)

    def list_builds_since(self, timestamp):
        '''Return a list of new builds since the given timestamp'''
        return self.listBuilds(completeAfter=timestamp)

    def list_tags(self, nvr):
        '''Return a list of tags applied to the package with the given nvr'''
        return [t['name'] for t in self.listTags(nvr)]

    def tag_history(self, nvr):
        '''Returns a list of every tag every applied to the given nvr'''
        return [t['tag_name'] for t in self.tagHistory(build=nvr)]

    def latest_by_tag(self, tag, pkgname, max_evr=None):
        '''Get the latest package for the given name in the given tag. If you
        set max_nvr, it is *exclusive*.
        max_evr = (epoch, version, release)
        '''
        # allow epoch to be empty, transcode it to zero
        if max_evr and not max_evr[0]:
            max_evr = (0, max_evr[1], max_evr[2])

        if max_evr:
            pkgs = self.listTagged(tag, package=pkgname)
            # sort the packages from highest EVR to lowest
            pkgs.sort(cmp=lambda x, y: compareEVR(x, y), reverse=True)
            for pkg in pkgs[:]:
                if rpmUtils.miscutils.compareEVR((pkg['epoch'] or '0',
                pkg['version'], pkg['release']), max_evr) >= 0:
                    # remove all packages greater or equal than max_evr
                    pkgs.remove(pkg)
                else:
                    # since packages are sorted, we can stop here
                    break
        else:
            pkgs = self.listTagged(tag, package=pkgname, latest=True)
        if pkgs:
            # the winner is the first package in list
            return pkgs[0]

    def build_failed(self, nvr):
        return (self.getBuild(nvr)['state'] > 1)

    # Idea taken from the koji CLI
    def ensure_connection(self):
        '''Check the connection to koji.'''
        try:
            ret = self.getAPIVersion()
        except xmlrpclib.ProtocolError:
            print >> sys.stderr, "Unable to connect to server"
            sys.exit(2)
        if ret != koji.API_VERSION:
            print >> sys.stderr, "Warning: API mismatch (Server %d, client %d)" % (ret,koji.API_VERSION)

    def test_list_builds_since(self):
        builds = []
        starttime = time.time()
        timestamp = time.time()
        while not builds and (starttime - timestamp < 10000):
            timestamp -= 3000
            builds = self.list_builds_since(timestamp)
        print "Most recent builds: %.0f min ago" % ((starttime-timestamp)/60.0)
        return builds

    def list_previous_releases(self, name, tag, max_evr=None,
        unstable_tags=False):
        '''Get most recent releases of package in specified tag (inheritence
        enabled). Returns dictionary in the format {tag: NVR object}. Optional
        max_evr says that all returned release versions must be lesser then
        it. If you enable unstable_tags, then also 'testing' and 'candidate'
        tags are considered.
        max_evr = (epoch, version, release)
        '''
        prev_rlss = {}
        # minor hack - assume -candidate parents = stable+updates+testing
        # TODO: provide this info in repoinfo instead
        if tag.endswith('-updates-candidate'):
            testtag = tag.replace('-candidate','-testing')
            testrepo = repoinfo.getrepo_by_tag(testtag)
            # -testing parents = stable+updates. add -testing and we're good.
            parents = testrepo['parents'] + [testrepo['name']]
        else:
            parents = repoinfo.getrepo_by_tag(tag)['parents']
        if parents:
            for p in parents:
                ptag = repoinfo.get(p,'tag')
                if ('-testing' in ptag or '-candidate' in ptag) and \
                not unstable_tags:
                    # skip unstable tags
                    continue
                prev_rlss[ptag] = self.latest_by_tag(ptag, name, max_evr=max_evr)
        else:
            # NOTE: for tags with no parents (e.g. rawhide) we obviously can't
            # check the parent repos' tags to get the previously released
            # package NVRs. Instead we use koji's tagHistory command to find
            # the previous version of this package that had this tag.
            hist = self.tagHistory(tag=tag, package=name)
            # remove untagged builds
            hist = filter(lambda h: h['revoke_event'] is None, hist)
            # ensure sorting by time
            hist.sort(key=lambda h: h['create_ts'])
            # hist[-2] = not the currently-tagged build, the one before that
            prev_rlss[tag] = self.getBuild(hist[-2]['build_id'])
        return prev_rlss

    def list_previous_release(self, name, tag, max_evr=None,
        unstable_tags=False):
        ''' The same as list_previous_releases(), but returns a single NVR
        object (the most recent one) regardless of its tag.
        '''
        prev_rlss = self.list_previous_releases(name, tag, max_evr,
            unstable_tags)
        # take only NVR objects and erase possible None's
        prev_rlss = [rls for rls in prev_rlss.values() if rls]
        prev_rlss.sort(cmp=lambda x, y: compareEVR(x, y), reverse=True)
        if prev_rlss:
            return prev_rlss[0]

    def nvr_to_urls(self, nvr, arches=None, debuginfo=False):
        info = self.getBuild(nvr)
        baseurl = '/'.join((pkgurl, info['package_name'],
                           info['version'], info['release']))

        rpms = self.listRPMs(buildID=info['id'], arches=arches)
        if not debuginfo:
            rpms = filter(lambda r: not r['name'].endswith('-debuginfo'), rpms)
        # XXX option to skip srpms too?

        return ['%s/%s' % (baseurl, koji.pathinfo.rpm(r)) for r in rpms]

    def nvr_to_rpms(self, nvr, debuginfo=False, src=True):
        ''' Get a list of RPMs existing for specified NVR. All RPMs have 'url'
        key added with URL where to download the RPM as the value.'''

        info = self.getBuild(nvr)
        baseurl = '/'.join((pkgurl, info['package_name'],
                           info['version'], info['release']))

        rpms = self.listRPMs(buildID=info['id'])
        if not debuginfo:
            rpms = filter(lambda r: not r['name'].endswith('-debuginfo'), rpms)
        if not src:
            rpms = filter(lambda r: not r['arch'] == 'src', rpms)

        for r in rpms:
            r['url'] = '%s/%s' % (baseurl, koji.pathinfo.rpm(r))

        return rpms


def compareEVR(nvr1, nvr2):
    ''' Compare EVR from two NVR objects. '''
    return rpmUtils.miscutils.compareEVR((nvr1['epoch'] or '0',
        nvr1['version'], nvr1['release']), (nvr2['epoch'] or '0',
        nvr2['version'], nvr2['release']))

