#
# Rawhide Acceptance Test Suite - common functions
#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import os
import sys
import gzip
import hashlib
import datetime
import urlgrabber.grabber
import urlgrabber.progress
import config
import urllib
import socket

def timestamp_to_compose_id(timestamp=None, serial=1):
    if not timestamp:
        d = datetime.datetime.today()
    else:
        d = datetime.datetime.fromtimestamp(timestamp)
    return int(d.strftime('%Y%m%d') + '%02u' % serial)

def todays_compose_id(serial=1):
    return timestamp_to_compose_id(serial=serial)

def get_basearch():
    '''like get_arch, but returns the basearch (as used by yum etc.)'''
    arch = os.uname()[4]
    if arch in ('i486', 'i586', 'i686'):
        arch = 'i386'
    elif arch == 'ppc64':
        arch = 'ppc'
    return arch

def safe_makedirs(fullpath):
    try:
        os.makedirs(fullpath)
    except OSError, e:
        if e.errno != 17: # already exists
            raise

def ishexstring(string):
    if string.startswith('0x') or string.startswith('0X'):
        string = string[2:]
    return string.strip('abcdefABCDEF0123456789') == ''

def nonempty(filename):
    return os.stat(filename)[6] > 0

def do_gunzip(infile, outfile):
    _write_file(gzip.open(infile), open(outfile,"wb"))

def do_gzip(infile, outfile):
    _write_file(open(infile), gzip.open(outfile,'wb'))

def _write_file(infile, outfile):
    while True:
        data = infile.read(config.BLOCKSIZE)
        if data:
            outfile.write(data)
        else:
            break
    infile.close()
    outfile.close()

def checkhash(filename, algo, hexdigest):
    hasher = hashlib.new(algo)
    f = open(filename)
    data = f.read(config.BLOCKSIZE)
    while data:
        hasher.update(data)
        data = f.read(config.BLOCKSIZE)
    hasher.update(data)
    f.close()
    r = check("%s %s" % (os.path.basename(filename), algo),
              hasher.hexdigest() == hexdigest)
    return r

def check_opt_in(pkgname, dist_release):
    """take a package name and distribution release and check for an autoqa-optin file"""

    url = 'http://fedorapeople.org/packages/'
    fullurl = url + '%s/%s/%s/autoqa-optin' % (pkgname[0].lower(), pkgname, dist_release)
    tmout = socket.getdefaulttimeout()
    socket.setdefaulttimeout(10) # if it takes more than that, just consider as not opted in
    try:
        h = urllib.urlopen(fullurl)
    except IOError, e:
        return False
    finally:
        socket.setdefaulttimeout(tmout)

    return h.getcode() == 200 # file exists (HTTP 200 OK)


grabber = urlgrabber.grabber.URLGrabber()
if hasattr(sys.stdout, "fileno") and os.isatty(sys.stdout.fileno()):
    grabber.opts.progress_obj = urlgrabber.progress.TextMeter()

# TODO: keep a list of downloaded files
def simplegrab(relpath):
    fullpath = os.path.join(config.scratchdir,relpath)
    safe_makedirs(os.path.dirname(fullpath))
    return grabber.urlgrab(str(relpath), fullpath)

def openlog(logname):
    '''return open file object for logdir/logname. truncates existing files.'''
    fullname = os.path.join(config.logdir, logname)
    safe_makedirs(os.path.dirname(fullname))
    return open(fullname, "wb")

def writelog(logname, logdata):
    '''open the given logname (see openlog) and write the given data.'''
    log = openlog(logname)
    log.write(logdata)
    log.close()

def write_keyval(key, val, kvdir=config.keyvaldir):
    '''write val\n to keyvaldir/key'''
    fullname = os.path.join(kvdir, key)
    safe_makedirs(os.path.dirname(fullname))
    f = open(fullname, "wb")
    f.write("%s\n" % str(val))
    f.close()

def read_keyvals(kvdir=config.keyvaldir):
    keyvals = {}
    if os.path.isdir(kvdir):
        for filename in os.listdir(kvdir):
            f = open(os.path.join(kvdir,filename))
            val = f.read()
            f.close()
            keyvals[filename] = val.strip()
    return keyvals

class CheckFailedException(Exception):
    pass

def check(msg, ok):
    if ok:
        print "CHECK: %s: OK" % msg
        return True
    else:
        print "CHECK: %s: FAIL" % msg
        raise CheckFailedException

def test(key, result):
    if result:
        write_keyval(key, "OK")
        print "TEST RESULT: %s: OK" % key
        return result
    else:
        write_keyval(key, "FAIL")
        print "TEST RESULT: %s: FAIL" % key
        raise CheckFailedException

def make_autotest_url(config):
    autotest_server = config.get('general', 'autotest_server')
    jobtag = config.get('job', 'tag')
    autotest_url = "http://%s/results/%s/" % (autotest_server, jobtag)
    return(autotest_url)


