# -*- coding: utf-8 -*-
#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Kamil Paral <kparal@redhat.com>

from autotest_lib.client.bin import test, utils
from autotest_lib.client.bin.test_config import config_loader
from autotest_lib.client.common_lib import error
import autoqa.koji_utils
import autoqa.util
from autoqa.repoinfo import repoinfo
import os
import rpmUtils.miscutils

class rpmguard(test.test):
    version = 1 # increment if setup() changes

    def setup(self):
        # Download latest rawhide rpmlint because rpmlint in F12 (ver 0.91)
        # is too old
        rpmlint = utils.system_output("koji latest-pkg dist-f13 --quiet \
rpmlint | cut -d ' ' -f 1", retain_output=True)
        utils.system('cd %s; koji download-build --arch noarch \
--latestfrom dist-f13 rpmlint' % self.tmpdir)
        utils.system('yum -y localinstall --nogpgcheck %s/%s.noarch.rpm' \
            % (self.tmpdir, rpmlint))


    def initialize(self, envr, name, kojitag, config):
        self.config = config_loader(config, self.tmpdir)
        # we have several results (comparing different architectures), let's
        # store it separately
        self.results = []
        # temporary results for each architecture
        self.arch_results = {}
        self.rpmguard = os.path.join(self.bindir, 'rpmguard')
        self.rpmdir = os.path.join(self.tmpdir, 'rpms')
        #URL of logs/results stored on autotest-server
        self.autotest_url = autoqa.util.make_autotest_url(self.config)
        os.makedirs(self.rpmdir)


    def run_once(self, envr, name, kojitag):
        # where to log output
        out = open(os.path.join(self.resultsdir,'rpmguard.log'),'wb')

        koji = autoqa.koji_utils.SimpleKojiClientSession()
        # get the most recent release available
        # add .noarch to parse filename correctly
        nvrea = rpmUtils.miscutils.splitFilename(envr + '.noarch')
        lastBuild = koji.list_previous_release(name, kojitag,
            max_evr=(nvrea[3], nvrea[1], nvrea[2]))
        # if there is no such build, we don't have anything to compare
        if not lastBuild:
            warn = "N: There is no previous build of %s in parents of \
%s tag." % (envr, kojitag)
            print warn
            out.write(warn + '\n')
            out.close()
            return

        # now we need list of RPMs available for each build
        new_rpms = koji.nvr_to_rpms(envr, src=False)
        old_rpms = koji.nvr_to_rpms(lastBuild['nvr'], src=False)
        # and match the RPMs according to build name and architecture as
        # (old one, new one)
        rpm_to_match = []

        # traverse all RPMs in the newer build
        for new_rpm in new_rpms:
            # find the matching older RPM, there should be exactly one
            old_rpm = [r for r in old_rpms if r['arch'] == new_rpm['arch']
                        and r['name'] == new_rpm['name']]
            # we should mark oldere these packages as already matched
            for o in old_rpm:
                o['seen'] = True
            # older RPM may not exist
            if not old_rpm:
                err = 'N: There is no previous RPM for recent NVRA: %s.%s' % \
                        (new_rpm['nvr'], new_rpm['arch'])
                print err
                out.write(err + '\n')
                continue
            # there certainly should be more than 1 matching older RPM, but
            # let's check as a precaution
            if len(old_rpm) > 1:
                err = 'N: There are %d existing older RPMs for NVRA: %s.%s' % \
                        (len(old_rpm), new_rpm['nvr'], new_rpm['arch'])
                print err
                out.write(err + '\n')
                continue

            # now there is certainly one older RPM matching the newer one
            old_rpm = old_rpm[0]
            rpm_to_match.append((old_rpm, new_rpm))

        # there may be some older RPMs not longer available in the newer build
        for old_rpm in old_rpms:
            if not old_rpm.has_key('seen'):
                err = 'N: There is no new RPM to match older NVRA: \
%s.%s' % (old_rpm['nvr'], old_rpm['arch'])
                print err
                out.write(err + '\n')

        # let's remember if any of invoked command failed
        test_result = 'PASS'

        # let's compare the packages finally
        for old_rpm, new_rpm in rpm_to_match:
            # fetch old and new rpms to rpmdir
            url = old_rpm['url']
            print "Grabbing %s" % url
            old_file = os.path.join(self.rpmdir, os.path.basename(url))
            autoqa.util.grabber.urlgrab(url, old_file)
            url = new_rpm['url']
            print "Grabbing %s" % url
            new_file = os.path.join(self.rpmdir, os.path.basename(url))
            autoqa.util.grabber.urlgrab(url, new_file)

            # run rpmguard
            info = 'N: Comparing %s.%s and %s.%s ...' % \
             (old_rpm['nvr'], old_rpm['arch'], new_rpm['nvr'], new_rpm['arch'])
            print info
            cmd = '%s %s %s' % (self.rpmguard, old_file, new_file)
            result = utils.run(cmd+" 2>&1", ignore_status=True,
                stdout_tee=utils.TEE_TO_LOGS, stderr_tee=utils.TEE_TO_LOGS)
            # if no warnings or some warnings
            if result.exit_status == 0 or result.exit_status == 10:
                # store the results in temporary arch_results
                key = (old_rpm['nvr'],new_rpm['nvr'])
                if not self.arch_results.has_key(key):
                    self.arch_results[key] = {}
                archs = self.arch_results[key]
                archs[old_rpm['arch']] = result.stdout
            # if command failed
            else:
                test_result = 'FAIL'
                self.results.append(info + '\n' + result.stdout + '\n' + \
                    result.stderr + '\nN: ----')
            print 'N: ----'

        # Copy arch_results to results and eliminate duplicates
        for (old_nvr, new_nvr) in self.arch_results.keys():
            res_arch = {}
            for arch, result in \
            self.arch_results[(old_nvr, new_nvr)].iteritems():
                if not res_arch.has_key(result):
                    res_arch[result] = [arch]
                else:
                    res_arch[result].append(arch)
            for result, archs in res_arch.iteritems():
                archlist = ''
                for arch in archs:
                    archlist += arch + ', '
                archlist = archlist[:-2]
                info = 'N: Comparing %s and %s (archs: %s) ...\n' % \
                    (old_nvr, new_nvr, archlist)
                outro = 'N: ----'
                self.results.append(info + result + outro)

        # Write a copy of the output into the log
        log = '\n'.join(self.results) + '\n'
        out.write(log)
        out.close()

        # email results to mailing list and to pkg owner if they optin
        send_to = []
        result_email = self.config.get('test','result_email')
        if result_email and isinstance(result_email, str):
            send_to = result_email.split(',')
            collection_name = repoinfo.getrepo_by_tag(kojitag)['collection_name']
            if collection_name and autoqa.util.check_opt_in(name, collection_name):
                #FIXME - hardcoded partial email address here - obviously sub-par
                send_to.append('%s-owner@fedoraproject.org' % name)

        if send_to:
            warning_count = len([line for line in log.splitlines() \
                            if line.startswith('W: ')])
            print 'Sending email with results to %s ...' % result_email
            mail_body = "Stored logs available at <%s>\n\n" % self.autotest_url
            mail_body += log
            utils.send_email(mail_to=send_to,
                mail_from=self.config.get('test','mail_from'),
                mail_server=self.config.get('test','smtpserver'),
                subject='rpmguard: %u warnings for package %s' % \
                    (warning_count, envr),
                body = mail_body)

        # if some command failed, make this test fail
        if test_result == 'FAIL':
            raise error.TestFail
